# Copyright (c) 2016 EMC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""Driver for EMC CoprHD ScaleIO volumes."""

from oslo_config import cfg
from oslo_log import log as logging
import requests
import six
from six.moves import http_client
from six.moves import urllib

from cinder import exception
from cinder.i18n import _
from cinder import interface
from cinder.volume import configuration
from cinder.volume import driver
from cinder.volume.drivers.coprhd import common as coprhd_common
from cinder.volume import utils as volume_utils


LOG = logging.getLogger(__name__)

scaleio_opts = [
    cfg.StrOpt('coprhd_scaleio_rest_gateway_host',
               default='None',
               help='Rest Gateway IP or FQDN for Scaleio'),
    cfg.PortOpt('coprhd_scaleio_rest_gateway_port',
                default=4984,
                help='Rest Gateway Port for Scaleio'),
    cfg.StrOpt('coprhd_scaleio_rest_server_username',
               default=None,
               help='Username for Rest Gateway'),
    cfg.StrOpt('coprhd_scaleio_rest_server_password',
               default=None,
               help='Rest Gateway Password',
               secret=True),
    cfg.BoolOpt('scaleio_verify_server_certificate',
                default=False,
                help='verify server certificate'),
    cfg.StrOpt('scaleio_server_certificate_path',
               default=None,
               help='Server certificate path')
]

CONF = cfg.CONF
CONF.register_opts(scaleio_opts, group=configuration.SHARED_CONF_GROUP)


@interface.volumedriver
class EMCCoprHDScaleIODriver(driver.VolumeDriver):
    """CoprHD ScaleIO Driver."""
    VERSION = "3.0.0.0"
    server_token = None

    # ThirdPartySystems wiki page
    CI_WIKI_NAME = "EMC_CoprHD_CI"

    def __init__(self, *args, **kwargs):
        super(EMCCoprHDScaleIODriver, self).__init__(*args, **kwargs)
        self.configuration.append_config_values(scaleio_opts)
        self.common = self._get_common_driver()

    def _get_common_driver(self):
        return coprhd_common.EMCCoprHDDriverCommon(
            protocol='scaleio',
            default_backend_name=self.__class__.__name__,
            configuration=self.configuration)

    def check_for_setup_error(self):
        self.common.check_for_setup_error()
        if (self.configuration.scaleio_verify_server_certificate is True and
                self.configuration.scaleio_server_certificate_path is None):
            message = _("scaleio_verify_server_certificate is True but"
                        " scaleio_server_certificate_path is not provided"
                        " in cinder configuration")
            raise exception.VolumeBackendAPIException(data=message)

    def create_volume(self, volume):
        """Creates a Volume."""
        self.common.create_volume(volume, self, True)
        self.common.set_volume_tags(volume, ['_obj_volume_type'], True)
        vol_size = self._update_volume_size(int(volume.size))
        return {'size': vol_size}

    def _update_volume_size(self, vol_size):
        """update the openstack volume size."""
        default_size = 8
        if (vol_size % default_size) != 0:
            return (vol_size / default_size) * default_size + default_size
        else:
            return vol_size

    def create_cloned_volume(self, volume, src_vref):
        """Creates a cloned Volume."""
        self.common.create_cloned_volume(volume, src_vref, True)
        self.common.set_volume_tags(volume, ['_obj_volume_type'], True)

    def create_volume_from_snapshot(self, volume, snapshot):
        """Creates a volume from a snapshot."""
        self.common.create_volume_from_snapshot(snapshot, volume, True)
        self.common.set_volume_tags(volume, ['_obj_volume_type'], True)

    def extend_volume(self, volume, new_size):
        """expands the size of the volume."""
        self.common.expand_volume(volume, new_size)

    def delete_volume(self, volume):
        """Deletes an volume."""
        self.common.delete_volume(volume)

    def create_snapshot(self, snapshot):
        """Creates a snapshot."""
        self.common.create_snapshot(snapshot, True)

    def delete_snapshot(self, snapshot):
        """Deletes a snapshot."""
        self.common.delete_snapshot(snapshot)

    def ensure_export(self, context, volume):
        """Driver entry point to get the export info for an existing volume."""
        pass

    def create_export(self, context, volume, connector=None):
        """Driver entry point to get the export info for a new volume."""
        pass

    def remove_export(self, context, volume):
        """Driver exntry point to remove an export for a volume."""
        pass

    def create_group(self, context, group):
        """Creates a group."""
        if volume_utils.is_group_a_cg_snapshot_type(group):
            return self.common.create_consistencygroup(context, group, True)

        # If the group is not consistency group snapshot enabled, then
        # we shall rely on generic volume group implementation
        raise NotImplementedError()

    def update_group(self, context, group, add_volumes=None,
                     remove_volumes=None):
        """Updates volumes in group."""
        if volume_utils.is_group_a_cg_snapshot_type(group):
            return self.common.update_consistencygroup(group, add_volumes,
                                                       remove_volumes)

        # If the group is not consistency group snapshot enabled, then
        # we shall rely on generic volume group implementation
        raise NotImplementedError()

    def create_group_from_src(self, ctxt, group, volumes,
                              group_snapshot=None, snapshots=None,
                              source_group=None, source_vols=None):
        """Creates a group from source."""
        if volume_utils.is_group_a_cg_snapshot_type(group):
            message = _("create group from source is not supported "
                        "for CoprHD if the group type supports "
                        "consistent group snapshot.")
            raise exception.VolumeBackendAPIException(data=message)
        else:
            raise NotImplementedError()

    def delete_group(self, context, group, volumes):
        """Deletes a group."""
        if volume_utils.is_group_a_cg_snapshot_type(group):
            return self.common.delete_consistencygroup(context, group,
                                                       volumes, True)

        # If the group is not consistency group snapshot enabled, then
        # we shall rely on generic volume group implementation
        raise NotImplementedError()

    def create_group_snapshot(self, context, group_snapshot, snapshots):
        """Creates a group snapshot."""
        if volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
            LOG.debug("creating a group snapshot")
            return self.common.create_cgsnapshot(group_snapshot, snapshots,
                                                 True)

        # If the group is not consistency group snapshot enabled, then
        # we shall rely on generic volume group implementation
        raise NotImplementedError()

    def delete_group_snapshot(self, context, group_snapshot, snapshots):
        """Deletes a group snapshot."""
        if volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
            return self.common.delete_cgsnapshot(group_snapshot, snapshots,
                                                 True)

        # If the group is not consistency group snapshot enabled, then
        # we shall rely on generic volume group implementation
        raise NotImplementedError()

    def check_for_export(self, context, volume_id):
        """Make sure volume is exported."""
        pass

    def initialize_connection(self, volume, connector):
        """Initializes the connection and returns connection info."""

        volname = self.common._get_resource_name(volume,
                                                 coprhd_common.MAX_SIO_LEN,
                                                 True)

        properties = {}
        properties['scaleIO_volname'] = volname
        properties['scaleIO_volume_id'] = volume.provider_id
        properties['hostIP'] = connector['ip']
        properties[
            'serverIP'] = self.configuration.coprhd_scaleio_rest_gateway_host
        properties[
            'serverPort'] = self.configuration.coprhd_scaleio_rest_gateway_port
        properties[
            'serverUsername'] = (
            self.configuration.coprhd_scaleio_rest_server_username)
        properties[
            'serverPassword'] = (
            self.configuration.coprhd_scaleio_rest_server_password)
        properties['iopsLimit'] = None
        properties['bandwidthLimit'] = None
        properties['serverToken'] = self.server_token

        initiator_ports = []
        initiator_port = self._get_client_id(properties['serverIP'],
                                             properties['serverPort'],
                                             properties['serverUsername'],
                                             properties['serverPassword'],
                                             properties['hostIP'])
        initiator_ports.append(initiator_port)

        properties['serverToken'] = self.server_token
        self.common.initialize_connection(volume,
                                          'scaleio',
                                          initiator_ports,
                                          connector['host'])

        dictobj = {
            'driver_volume_type': 'scaleio',
            'data': properties,
        }

        return dictobj

    def terminate_connection(self, volume, connector, **kwargs):
        """Disallow connection from connector."""

        volname = volume.display_name
        properties = {}
        properties['scaleIO_volname'] = volname
        properties['scaleIO_volume_id'] = volume.provider_id
        properties['hostIP'] = connector['ip']
        properties[
            'serverIP'] = self.configuration.coprhd_scaleio_rest_gateway_host
        properties[
            'serverPort'] = self.configuration.coprhd_scaleio_rest_gateway_port
        properties[
            'serverUsername'] = (
            self.configuration.coprhd_scaleio_rest_server_username)
        properties[
            'serverPassword'] = (
            self.configuration.coprhd_scaleio_rest_server_password)
        properties['serverToken'] = self.server_token

        initiator_port = self._get_client_id(properties['serverIP'],
                                             properties['serverPort'],
                                             properties['serverUsername'],
                                             properties['serverPassword'],
                                             properties['hostIP'])
        init_ports = []
        init_ports.append(initiator_port)
        self.common.terminate_connection(volume,
                                         'scaleio',
                                         init_ports,
                                         connector['host'])

    def get_volume_stats(self, refresh=False):
        """Get volume status.

        If 'refresh' is True, run update the stats first.
        """
        if refresh:
            self.update_volume_stats()

        return self._stats

    def update_volume_stats(self):
        """Retrieve stats info from virtual pool/virtual array."""
        LOG.debug("Updating volume stats")
        self._stats = self.common.update_volume_stats()

    def _get_client_id(self, server_ip, server_port, server_username,
                       server_password, sdc_ip):
        ip_encoded = urllib.parse.quote(sdc_ip, '')
        ip_double_encoded = urllib.parse.quote(ip_encoded, '')

        request = ("https://%s:%s/api/types/Sdc/instances/getByIp::%s/" %
                   (server_ip, six.text_type(server_port), ip_double_encoded))

        LOG.info("ScaleIO get client id by ip request: %s", request)

        if self.configuration.scaleio_verify_server_certificate:
            verify_cert = self.configuration.scaleio_server_certificate_path
        else:
            verify_cert = False

        r = requests.get(
            request, auth=(server_username, self.server_token),
            verify=verify_cert)
        r = self._check_response(
            r, request, server_ip, server_port,
            server_username, server_password)

        sdc_id = r.json()
        if not sdc_id:
            msg = (_("Client with ip %s wasn't found ") % sdc_ip)
            LOG.error(msg)
            raise exception.VolumeBackendAPIException(data=msg)
        if r.status_code != http_client.OK and "errorCode" in sdc_id:
            msg = (_("Error getting sdc id from ip %(sdc_ip)s:"
                     " %(sdc_id_message)s") % {'sdc_ip': sdc_ip,
                                               'sdc_id_message': sdc_id[
                                                   'message']})
            LOG.error(msg)
            raise exception.VolumeBackendAPIException(data=msg)
        LOG.info("ScaleIO sdc id is %s", sdc_id)
        return sdc_id

    def _check_response(self, response, request,
                        server_ip, server_port,
                        server_username, server_password):
        if (response.status_code == http_client.UNAUTHORIZED) or (
                response.status_code == http_client.FORBIDDEN):
            LOG.info(
                "Token is invalid, going to re-login and get a new one")

            login_request = ("https://%s:%s/api/login" %
                             (server_ip, six.text_type(server_port)))
            if self.configuration.scaleio_verify_server_certificate:
                verify_cert = (
                    self.configuration.scaleio_server_certificate_path)
            else:
                verify_cert = False

            r = requests.get(
                login_request, auth=(server_username, server_password),
                verify=verify_cert)

            token = r.json()
            self.server_token = token
            # repeat request with valid token
            LOG.info("Going to perform request again %s with valid token",
                     request)
            res = requests.get(
                request, auth=(server_username, self.server_token),
                verify=verify_cert)
            return res
        return response

    def retype(self, ctxt, volume, new_type, diff, host):
        """Change the volume type."""
        return self.common.retype(ctxt, volume, new_type, diff, host)



# Copyright 2016 Nexenta Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Unit tests for OpenStack Cinder volume driver
"""

import mock
from mock import patch
from oslo_utils import units

from cinder import context
from cinder import db
from cinder import exception
from cinder import test
from cinder.volume import configuration as conf
from cinder.volume.drivers.nexenta.ns5 import iscsi
from cinder.volume.drivers.nexenta.ns5 import jsonrpc


class TestNexentaISCSIDriver(test.TestCase):
    TEST_VOLUME_NAME = 'volume1'
    TEST_VOLUME_NAME2 = 'volume2'
    TEST_VOLUME_NAME3 = 'volume3'
    TEST_SNAPSHOT_NAME = 'snapshot1'
    TEST_VOLUME_REF = {
        'name': TEST_VOLUME_NAME,
        'size': 1,
        'id': '1',
        'status': 'available'
    }
    TEST_VOLUME_REF2 = {
        'name': TEST_VOLUME_NAME2,
        'size': 1,
        'id': '2',
        'status': 'in-use'
    }
    TEST_VOLUME_REF3 = {
        'name': TEST_VOLUME_NAME3,
        'size': 2,
        'id': '2',
        'status': 'in-use'
    }
    TEST_SNAPSHOT_REF = {
        'name': TEST_SNAPSHOT_NAME,
        'volume_name': TEST_VOLUME_NAME,
        'volume_id': '1',
        'volume_size': 1
    }

    def __init__(self, method):
        super(TestNexentaISCSIDriver, self).__init__(method)

    def setUp(self):
        super(TestNexentaISCSIDriver, self).setUp()
        self.cfg = mock.Mock(spec=conf.Configuration)
        self.ctxt = context.get_admin_context()
        self.cfg.nexenta_dataset_description = ''
        self.cfg.nexenta_host = '1.1.1.1'
        self.cfg.nexenta_user = 'admin'
        self.cfg.nexenta_password = 'nexenta'
        self.cfg.nexenta_volume = 'cinder'
        self.cfg.nexenta_rest_port = 2000
        self.cfg.nexenta_use_https = False
        self.cfg.nexenta_iscsi_target_portal_port = 8080
        self.cfg.nexenta_target_prefix = 'iqn:'
        self.cfg.nexenta_target_group_prefix = 'cinder/'
        self.cfg.nexenta_ns5_blocksize = 32
        self.cfg.nexenta_sparse = True
        self.cfg.nexenta_dataset_compression = 'on'
        self.cfg.nexenta_dataset_dedup = 'off'
        self.cfg.reserved_percentage = 20
        self.cfg.nexenta_volume = 'pool'
        self.cfg.nexenta_volume_group = 'dsg'
        self.nef_mock = mock.Mock()
        self.mock_object(jsonrpc, 'NexentaJSONProxy',
                         return_value=self.nef_mock)
        self.drv = iscsi.NexentaISCSIDriver(
            configuration=self.cfg)
        self.drv.db = db
        self.drv._fetch_volumes = lambda: None
        self.drv.do_setup(self.ctxt)

    def _create_volume_db_entry(self):
        vol = {
            'id': '1',
            'size': 1,
            'status': 'available',
            'provider_location': self.TEST_VOLUME_NAME
        }
        return db.volume_create(self.ctxt, vol)['id']

    def test_do_setup(self):
        self.nef_mock.post.side_effect = exception.NexentaException(
            'Could not create volume group')
        self.assertRaises(
            exception.NexentaException,
            self.drv.do_setup, self.ctxt)

        self.nef_mock.post.side_effect = exception.NexentaException(
            '{"code": "EEXIST"}')
        self.assertIsNone(self.drv.do_setup(self.ctxt))

    def test_check_for_setup_error(self):
        self.nef_mock.get.return_value = {
            'data': [{'name': 'iscsit', 'state': 'offline'}]}
        self.assertRaises(
            exception.NexentaException, self.drv.check_for_setup_error)

        self.nef_mock.get.side_effect = exception.NexentaException()
        self.assertRaises(LookupError, self.drv.check_for_setup_error)

    def test_create_volume(self):
        self.drv.create_volume(self.TEST_VOLUME_REF)
        url = 'storage/pools/pool/volumeGroups/dsg/volumes'
        self.nef_mock.post.assert_called_with(url, {
            'name': self.TEST_VOLUME_REF['name'],
            'volumeSize': 1 * units.Gi,
            'volumeBlockSize': 32768,
            'sparseVolume': self.cfg.nexenta_sparse})

    def test_delete_volume(self):
        self.drv.collect_zfs_garbage = lambda x: None
        self.nef_mock.delete.side_effect = exception.NexentaException(
            'Failed to destroy snapshot')
        self.assertIsNone(self.drv.delete_volume(self.TEST_VOLUME_REF))
        url = 'storage/pools/pool/volumeGroups'
        data = {'name': 'dsg', 'volumeBlockSize': 32768}
        self.nef_mock.post.assert_called_with(url, data)

    def test_extend_volume(self):
        self.drv.extend_volume(self.TEST_VOLUME_REF, 2)
        url = ('storage/pools/pool/volumeGroups/dsg/volumes/%(name)s') % {
            'name': self.TEST_VOLUME_REF['name']}
        self.nef_mock.put.assert_called_with(url, {
            'volumeSize': 2 * units.Gi})

    def test_delete_snapshot(self):
        self._create_volume_db_entry()
        url = ('storage/pools/pool/volumeGroups/dsg/'
               'volumes/volume-1/snapshots/snapshot1')

        self.nef_mock.delete.side_effect = exception.NexentaException('EBUSY')
        self.drv.delete_snapshot(self.TEST_SNAPSHOT_REF)
        self.nef_mock.delete.assert_called_with(url)

        self.nef_mock.delete.side_effect = exception.NexentaException('Error')
        self.drv.delete_snapshot(self.TEST_SNAPSHOT_REF)
        self.nef_mock.delete.assert_called_with(url)

    @patch('cinder.volume.drivers.nexenta.ns5.iscsi.'
           'NexentaISCSIDriver.create_snapshot')
    @patch('cinder.volume.drivers.nexenta.ns5.iscsi.'
           'NexentaISCSIDriver.delete_snapshot')
    @patch('cinder.volume.drivers.nexenta.ns5.iscsi.'
           'NexentaISCSIDriver.create_volume_from_snapshot')
    def test_create_cloned_volume(self, crt_vol, dlt_snap, crt_snap):
        self._create_volume_db_entry()
        vol = self.TEST_VOLUME_REF2
        src_vref = self.TEST_VOLUME_REF
        crt_vol.side_effect = exception.NexentaException()
        dlt_snap.side_effect = exception.NexentaException()
        self.assertRaises(
            exception.NexentaException,
            self.drv.create_cloned_volume, vol, src_vref)

    def test_create_snapshot(self):
        self._create_volume_db_entry()
        self.drv.create_snapshot(self.TEST_SNAPSHOT_REF)
        url = 'storage/pools/pool/volumeGroups/dsg/volumes/volume-1/snapshots'
        self.nef_mock.post.assert_called_with(
            url, {'name': 'snapshot1'})

    def test_create_larger_volume_from_snapshot(self):
        self._create_volume_db_entry()
        vol = self.TEST_VOLUME_REF3
        src_vref = self.TEST_SNAPSHOT_REF

        self.drv.create_volume_from_snapshot(vol, src_vref)

        # make sure the volume get extended!
        url = ('storage/pools/pool/volumeGroups/dsg/volumes/%(name)s') % {
            'name': self.TEST_VOLUME_REF3['name']}
        self.nef_mock.put.assert_called_with(url, {
            'volumeSize': 2 * units.Gi})

    def test_do_export(self):
        target_name = 'new_target'
        lun = 0

        class GetSideEffect(object):
            def __init__(self):
                self.lm_counter = -1

            def __call__(self, *args, **kwargs):
                # Find out whether the volume is exported
                if 'san/lunMappings?volume=' in args[0]:
                    self.lm_counter += 1
                    # a value for the first call
                    if self.lm_counter == 0:
                        return {'data': []}
                    else:
                        return {'data': [{'lun': lun}]}
                # Get the name of just created target
                elif 'san/iscsi/targets' in args[0]:
                    return {'data': [{'name': target_name}]}

        def post_side_effect(*args, **kwargs):
            if 'san/iscsi/targets' in args[0]:
                return {'data': [{'name': target_name}]}

        self.nef_mock.get.side_effect = GetSideEffect()
        self.nef_mock.post.side_effect = post_side_effect
        res = self.drv._do_export(self.ctxt, self.TEST_VOLUME_REF)
        provider_location = '%(host)s:%(port)s,1 %(name)s %(lun)s' % {
            'host': self.cfg.nexenta_host,
            'port': self.cfg.nexenta_iscsi_target_portal_port,
            'name': target_name,
            'lun': lun,
        }
        expected = {'provider_location': provider_location}
        self.assertEqual(expected, res)

    def test_remove_export(self):
        mapping_id = '1234567890'
        self.nef_mock.get.return_value = {'data': [{'id': mapping_id}]}
        self.drv.remove_export(self.ctxt, self.TEST_VOLUME_REF)
        url = 'san/lunMappings/%s' % mapping_id
        self.nef_mock.delete.assert_called_with(url)

    def test_update_volume_stats(self):
        self.nef_mock.get.return_value = {
            'bytesAvailable': 10 * units.Gi,
            'bytesUsed': 2 * units.Gi
        }
        location_info = '%(driver)s:%(host)s:%(pool)s/%(group)s' % {
            'driver': self.drv.__class__.__name__,
            'host': self.cfg.nexenta_host,
            'pool': self.cfg.nexenta_volume,
            'group': self.cfg.nexenta_volume_group,
        }
        stats = {
            'vendor_name': 'Nexenta',
            'dedup': self.cfg.nexenta_dataset_dedup,
            'compression': self.cfg.nexenta_dataset_compression,
            'description': self.cfg.nexenta_dataset_description,
            'driver_version': self.drv.VERSION,
            'storage_protocol': 'iSCSI',
            'total_capacity_gb': 10,
            'free_capacity_gb': 8,
            'reserved_percentage': self.cfg.reserved_percentage,
            'QoS_support': False,
            'volume_backend_name': self.drv.backend_name,
            'location_info': location_info,
            'iscsi_target_portal_port': (
                self.cfg.nexenta_iscsi_target_portal_port),
            'nef_url': self.drv.nef.url
        }
        self.drv._update_volume_stats()
        self.assertEqual(stats, self.drv._stats)



# Copyright (c) 2013 - 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Unit Tests for Huawei FusionStorage drivers.
"""

import mock
from oslo_config import cfg
from oslo_service import loopingcall

from cinder import context
from cinder import exception
from cinder.image import image_utils
from cinder import test
from cinder.tests.unit import fake_constants
from cinder.tests.unit import fake_snapshot
from cinder.tests.unit import fake_volume
from cinder.volume import configuration as conf
from cinder.volume.drivers.fusionstorage import dsware
from cinder.volume.drivers.fusionstorage import fspythonapi


test_volume = {'name': 'test_vol1',
               'size': 4,
               'volume_metadata': '',
               'host': 'host01@dsware',
               'instance_uuid': None,
               'provider_id': '127.0.0.1',
               'id': fake_constants.VOLUME_ID}

test_src_volume = {'name': 'test_vol2',
                   'size': 4,
                   'status': 'available'}

test_snapshot = {
    'name': 'test_snapshot1',
    'volume_id': fake_constants.VOLUME_ID,
    'volume_size': '4'}


class FakeDSWAREDriver(dsware.DSWAREDriver):
    def __init__(self):
        configuration = conf.Configuration(
            [
                cfg.StrOpt('fake'),
            ],
            None
        )
        super(FakeDSWAREDriver, self).__init__(configuration=configuration)
        self.dsware_client = fspythonapi.FSPythonApi()
        self.manage_ip = '127.0.0.1'
        self.pool_type = '1'


class DSwareDriverTestCase(test.TestCase):
    def setUp(self):
        super(DSwareDriverTestCase, self).setUp()
        self.driver = FakeDSWAREDriver()
        self.context = context.get_admin_context()
        self.volume = fake_volume.fake_volume_obj(context=self.context,
                                                  **test_volume)
        self.scr_volume = fake_volume.fake_volume_obj(context=self.context,
                                                      **test_src_volume)
        self.snapshot = fake_snapshot.fake_snapshot_obj(context=self.context,
                                                        **test_snapshot)

    def test_private_get_dsware_manage_ip(self):
        retval = self.driver._get_dsware_manage_ip(self.volume)
        self.assertEqual('127.0.0.1', retval)

        test_volume_fail_dict = {'name': 'test_vol',
                                 'size': 4,
                                 'volume_metadata': '',
                                 'host': 'host01@dsware',
                                 'provider_id': None}
        test_volume_fail = fake_volume.fake_volume_obj(context=self.context,
                                                       **test_volume_fail_dict)
        self.assertRaises(exception.CinderException,
                          self.driver._get_dsware_manage_ip,
                          test_volume_fail)

    def test_private_get_poolid_from_host(self):
        retval = self.driver._get_poolid_from_host(
            'abc@fusionstorage_sas2copy#0')
        self.assertEqual('0', retval)

        retval = self.driver._get_poolid_from_host(
            'abc@fusionstorage_sas2copy@0')
        self.assertEqual(self.driver.pool_type, retval)

        retval = self.driver._get_poolid_from_host(None)
        self.assertEqual(self.driver.pool_type, retval)

    @mock.patch.object(fspythonapi.FSPythonApi, 'create_volume')
    @mock.patch.object(fspythonapi.FSPythonApi, 'query_dsware_version')
    @mock.patch.object(dsware.DSWAREDriver, '_get_poolid_from_host')
    def test_private_create_volume_old_version(self, mock_get_poolid,
                                               mock_query_dsware,
                                               mock_create_volume):
        # query_dsware_version return 1, old version
        mock_query_dsware.return_value = 1
        mock_create_volume.return_value = 0
        self.driver._create_volume(self.volume.name,
                                   self.volume.size,
                                   True,
                                   'abc@fusionstorage_sas2copy')
        mock_create_volume.assert_called_with(self.volume.name, 0,
                                              self.volume.size, 1)

        self.driver._create_volume(self.volume.name,
                                   self.volume.size,
                                   False,
                                   'abc@fusionstorage_sas2copy')
        mock_create_volume.assert_called_with(self.volume.name, 0,
                                              self.volume.size, 0)

    @mock.patch.object(fspythonapi.FSPythonApi, 'create_volume')
    @mock.patch.object(fspythonapi.FSPythonApi, 'query_dsware_version')
    @mock.patch.object(dsware.DSWAREDriver, '_get_poolid_from_host')
    def test_private_create_volume_new_version(self, mock_get_poolid,
                                               mock_query_dsware,
                                               mock_create_volume):
        # query_dsware_version return 0, new version
        mock_query_dsware.return_value = 0
        mock_get_poolid.return_value = 0
        mock_create_volume.return_value = 0
        self.driver._create_volume(self.volume.name,
                                   self.volume.size,
                                   True,
                                   'abcE@fusionstorage_sas2copy#0')
        mock_create_volume.assert_called_with(self.volume.name, 0,
                                              self.volume.size, 1)

        self.driver._create_volume(self.volume.name,
                                   self.volume.size,
                                   False,
                                   'abc@fusionstorage_sas2copy#0')
        mock_create_volume.assert_called_with(self.volume.name, 0,
                                              self.volume.size, 0)

        mock_query_dsware.return_value = 0
        mock_get_poolid.return_value = 1
        mock_create_volume.return_value = 0
        self.driver._create_volume(self.volume.name,
                                   self.volume.size,
                                   True,
                                   'abc@fusionstorage_sas2copy#1')
        mock_create_volume.assert_called_with(self.volume.name, 1,
                                              self.volume.size, 1)

        self.driver._create_volume(self.volume.name,
                                   self.volume.size,
                                   False,
                                   'abc@fusionstorage_sas2copy#1')
        mock_create_volume.assert_called_with(self.volume.name, 1,
                                              self.volume.size, 0)

    @mock.patch.object(fspythonapi.FSPythonApi, 'create_volume')
    @mock.patch.object(fspythonapi.FSPythonApi, 'query_dsware_version')
    @mock.patch.object(dsware.DSWAREDriver, '_get_poolid_from_host')
    def test_private_create_volume_query_version_fail(self, mock_get_poolid,
                                                      mock_query_dsware,
                                                      mock_create_volume):
        # query_dsware_version return 500015, query dsware version failed!
        mock_query_dsware.return_value = 500015
        self.assertRaises(exception.CinderException,
                          self.driver._create_volume,
                          self.volume.name,
                          self.volume.size,
                          True,
                          'abc@fusionstorage_sas2copy#0')
        self.assertRaises(exception.CinderException,
                          self.driver._create_volume,
                          self.volume.name,
                          self.volume.size,
                          False,
                          'abc@fusionstorage_sas2copy#0')

    @mock.patch.object(fspythonapi.FSPythonApi, 'create_volume')
    @mock.patch.object(fspythonapi.FSPythonApi, 'query_dsware_version')
    @mock.patch.object(dsware.DSWAREDriver, '_get_poolid_from_host')
    def test_private_create_volume_fail(self, mock_get_poolid,
                                        mock_query_dsware,
                                        mock_create_volume):
        mock_query_dsware.return_value = 1
        # create_volume return 1, create volume failed
        mock_create_volume.return_value = 1
        self.assertRaises(exception.CinderException,
                          self.driver._create_volume,
                          self.volume.name,
                          self.volume.size,
                          True,
                          'abc@fusionstorage_sas2copy#0')
        self.assertRaises(exception.CinderException,
                          self.driver._create_volume,
                          self.volume.name,
                          self.volume.size,
                          False,
                          'abc@fusionstorage_sas2copy#0')

    @mock.patch.object(dsware.DSWAREDriver, '_create_volume')
    @mock.patch.object(fspythonapi.FSPythonApi, 'get_manage_ip')
    def test_create_volume(self, mock_get_manage_ip, mock_create_volume):
        # success
        mock_get_manage_ip.return_value = self.driver.manage_ip
        retval = self.driver.create_volume(self.volume)
        self.assertEqual({"provider_id": self.driver.manage_ip},
                         retval)

        # failure
        mock_create_volume.side_effect = exception.CinderException(
            'DSWARE Create Volume failed!')

        self.assertRaises(exception.CinderException,
                          self.driver.create_volume,
                          self.volume)

    @mock.patch.object(fspythonapi.FSPythonApi, 'create_volume_from_snap')
    def test_private_create_volume_from_snap(self, mock_create_volume):
        mock_create_volume.side_effect = [0, 1]
        self.driver._create_volume_from_snap(self.volume.name,
                                             self.volume.size,
                                             self.snapshot.name)
        # failure
        self.assertRaises(exception.CinderException,
                          self.driver._create_volume_from_snap,
                          self.volume.name, self.volume.size,
                          self.snapshot.name)

    @mock.patch.object(fspythonapi.FSPythonApi, 'extend_volume')
    def test_extend_volume(self, mock_extend_volume):
        mock_extend_volume.return_value = 0
        self.driver.extend_volume(self.volume, 5)

        mock_extend_volume.return_value = 0
        self.assertRaises(exception.CinderException,
                          self.driver.extend_volume,
                          self.volume,
                          3)

        mock_extend_volume.return_value = 1
        self.assertRaises(exception.CinderException,
                          self.driver.extend_volume,
                          self.volume,
                          5)

    @mock.patch.object(dsware.DSWAREDriver, '_create_volume_from_snap')
    @mock.patch.object(fspythonapi.FSPythonApi, 'get_manage_ip')
    def test_create_volume_from_snap(self, mock_manage_ip, mock_create_vol):
        # success
        mock_manage_ip.return_value = self.driver.manage_ip
        retval = self.driver.create_volume_from_snapshot(self.volume,
                                                         self.snapshot)
        self.assertEqual({"provider_id": self.driver.manage_ip},
                         retval)

        # failure
        mock_create_vol.side_effect = exception.CinderException(
            'DSWARE:create volume from snap failed')
        self.assertRaises(exception.CinderException,
                          self.driver.create_volume_from_snapshot,
                          self.volume, self.snapshot)

    @mock.patch.object(fspythonapi.FSPythonApi, 'create_volume_from_volume')
    @mock.patch.object(fspythonapi.FSPythonApi, 'get_manage_ip')
    @mock.patch.object(dsware.DSWAREDriver,
                       '_wait_for_create_cloned_volume_finish_timer')
    def test_create_cloned_volume(self, mock_wait_finish,
                                  mock_get_manage_ip, mock_create_volume):
        # success
        mock_create_volume.return_value = None
        mock_get_manage_ip.return_value = self.driver.manage_ip
        mock_wait_finish.return_value = True
        retval = self.driver.create_cloned_volume(self.volume, self.scr_volume)
        self.assertEqual({"provider_id": "127.0.0.1"}, retval)

        # failure:create exception
        mock_create_volume.return_value = 500015
        self.assertRaises(exception.CinderException,
                          self.driver.create_cloned_volume,
                          self.volume, self.scr_volume)
        # failure:wait exception
        mock_create_volume.return_value = None
        mock_wait_finish.return_value = False
        self.assertRaises(exception.CinderException,
                          self.driver.create_cloned_volume,
                          self.volume, self.scr_volume)

    @mock.patch.object(fspythonapi.FSPythonApi, 'query_volume')
    def test_private_check_create_cloned_volume_finish(self,
                                                       mock_query_volume):
        query_result_done = {'result': 0, 'vol_name': 'vol1',
                             'father_name': 'vol1_father', 'status': '0',
                             'vol_size': '1024', 'real_size': '1024',
                             'pool_id': 'pool1', 'create_time': '01/01/2015'}

        query_result_doing = {'result': 0, 'vol_name': 'vol1',
                              'father_name': 'vol1_father', 'status': '6',
                              'vol_size': '1024', 'real_size': '1024',
                              'pool_id': 'pool1', 'create_time': '01/01/2015'}

        mock_query_volume.side_effect = [
            query_result_done, query_result_doing, query_result_doing]

        # success
        self.assertRaises(loopingcall.LoopingCallDone,
                          self.driver._check_create_cloned_volume_finish,
                          self.volume.name)

        # in the process of creating volume
        self.driver.count = self.driver.configuration.clone_volume_timeout - 1
        self.driver._check_create_cloned_volume_finish(self.volume.name)
        self.assertEqual(self.driver.configuration.clone_volume_timeout,
                         self.driver.count)

        # timeout
        self.driver.count = self.driver.configuration.clone_volume_timeout
        self.assertRaises(loopingcall.LoopingCallDone,
                          self.driver._check_create_cloned_volume_finish,
                          self.volume.name)

    @mock.patch.object(dsware.DSWAREDriver,
                       '_check_create_cloned_volume_finish')
    def test_private_wait_for_create_cloned_volume_finish_timer(self,
                                                                mock_check):
        mock_check.side_effect = [loopingcall.LoopingCallDone(retvalue=True),
                                  loopingcall.LoopingCallDone(retvalue=False)]
        retval = self.driver._wait_for_create_cloned_volume_finish_timer(
            self.volume.name)
        self.assertTrue(retval)

        retval = self.driver._wait_for_create_cloned_volume_finish_timer(
            self.volume.name)
        self.assertFalse(retval)

    def test_private_analyse_output(self):
        out = 'ret_code=10\nret_desc=test\ndev_addr=/sda\n'
        retval = self.driver._analyse_output(out)
        self.assertEqual({'dev_addr': '/sda',
                          'ret_desc': 'test', 'ret_code': '10'},
                         retval)

        out = 'abcdefg'
        retval = self.driver._analyse_output(out)
        self.assertEqual({}, retval)

    def test_private_attach_volume(self):
        success = ['ret_code=0\nret_desc=success\ndev_addr=/dev/sdb\n', '']
        failure = ['ret_code=50510011\nret_desc=failed\ndev_addr=/dev/sdb\n',
                   '']
        mock_execute = self.mock_object(self.driver, '_execute')
        mock_execute.side_effect = [success, failure]
        # attached successful
        retval = self.driver._attach_volume(self.volume.name,
                                            self.driver.manage_ip)
        self.assertEqual({'dev_addr': '/dev/sdb',
                          'ret_desc': 'success', 'ret_code': '0'},
                         retval)
        # attached failure
        retval = self.driver._attach_volume(self.volume.name,
                                            self.driver.manage_ip)
        self.assertEqual({'dev_addr': '/dev/sdb',
                          'ret_desc': 'failed', 'ret_code': '50510011'},
                         retval)

    def test_private_detach_volume(self):
        success = ['ret_code=0\nret_desc=success\ndev_addr=/dev/sdb\n', '']
        failure = ['ret_code=50510011\nret_desc=failed\ndev_addr=/dev/sdb\n',
                   '']
        mock_execute = self.mock_object(self.driver, '_execute')
        mock_execute.side_effect = [success, failure]
        # detached successful
        retval = self.driver._detach_volume(self.volume.name,
                                            self.driver.manage_ip)
        self.assertEqual({'dev_addr': '/dev/sdb',
                          'ret_desc': 'success', 'ret_code': '0'},
                         retval)
        # detached failure
        retval = self.driver._detach_volume(self.volume.name,
                                            self.driver.manage_ip)
        self.assertEqual({'dev_addr': '/dev/sdb',
                          'ret_desc': 'failed',
                          'ret_code': '50510011'},
                         retval)

    def test_private_query_volume_attach(self):
        success = ['ret_code=0\nret_desc=success\ndev_addr=/dev/sdb\n', '']
        failure = ['ret_code=50510011\nret_desc=failed\ndev_addr=/dev/sdb\n',
                   '']
        mock_execute = self.mock_object(self.driver, '_execute')
        mock_execute.side_effect = [success, failure]
        # query successful
        retval = self.driver._query_volume_attach(self.volume.name,
                                                  self.driver.manage_ip)
        self.assertEqual({'dev_addr': '/dev/sdb',
                          'ret_desc': 'success',
                          'ret_code': '0'},
                         retval)
        # query failure
        retval = self.driver._query_volume_attach(self.volume.name,
                                                  self.driver.manage_ip)
        self.assertEqual({'dev_addr': '/dev/sdb',
                          'ret_desc': 'failed',
                          'ret_code': '50510011'},
                         retval)

    @mock.patch.object(dsware.DSWAREDriver, '_get_dsware_manage_ip')
    @mock.patch.object(dsware.DSWAREDriver, '_attach_volume')
    @mock.patch.object(image_utils, 'fetch_to_raw')
    @mock.patch.object(dsware.DSWAREDriver, '_detach_volume')
    def test_copy_image_to_volume(self, mock_detach, mock_fetch,
                                  mock_attach, mock_get_manage_ip):
        success = {'ret_code': '0',
                   'ret_desc': 'success',
                   'dev_addr': '/dev/sdb'}
        failure = {'ret_code': '50510011',
                   'ret_desc': 'failed',
                   'dev_addr': '/dev/sdb'}
        context = ''
        image_service = ''
        image_id = ''
        mock_get_manage_ip.return_value = '127.0.0.1'
        mock_attach.side_effect = [success, failure, success]
        mock_detach.side_effect = [success, failure, failure]

        # success
        self.driver.copy_image_to_volume(context, self.volume, image_service,
                                         image_id)

        # failure - attach failure
        self.assertRaises(exception.CinderException,
                          self.driver.copy_image_to_volume,
                          context, self.volume, image_service, image_id)

        # failure - detach failure
        self.assertRaises(exception.CinderException,
                          self.driver.copy_image_to_volume,
                          context, self.volume, image_service, image_id)

    @mock.patch.object(dsware.DSWAREDriver, '_get_dsware_manage_ip')
    @mock.patch.object(dsware.DSWAREDriver, '_attach_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_query_volume_attach')
    @mock.patch.object(image_utils, 'upload_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_detach_volume')
    def test_copy_volume_to_image_success(self, mock_detach, mock_upload,
                                          mock_query, mock_attach,
                                          mock_get_manage_ip):
        success = {'ret_code': '0',
                   'ret_desc': 'success',
                   'dev_addr': '/dev/sdb'}
        already_attached = {'ret_code': '50151401',
                            'ret_desc': 'already_attached',
                            'dev_addr': '/dev/sdb'}
        context = ''
        image_service = ''
        image_meta = ''

        mock_get_manage_ip.return_value = '127.0.0.1'
        mock_attach.return_value = success
        mock_detach.return_value = success
        self.driver.copy_volume_to_image(context, self.volume, image_service,
                                         image_meta)
        mock_upload.assert_called_with('', '', '', '/dev/sdb')

        mock_attach.return_value = already_attached
        mock_query.return_value = success
        mock_detach.return_value = success
        self.driver.copy_volume_to_image(context, self.volume, image_service,
                                         image_meta)
        mock_upload.assert_called_with('', '', '', '/dev/sdb')

    @mock.patch.object(dsware.DSWAREDriver, '_get_dsware_manage_ip')
    @mock.patch.object(dsware.DSWAREDriver, '_attach_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_query_volume_attach')
    @mock.patch.object(image_utils, 'upload_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_detach_volume')
    def test_copy_volume_to_image_attach_fail(self, mock_detach, mock_upload,
                                              mock_query, mock_attach,
                                              mock_get_manage_ip):
        failure = {'ret_code': '50510011',
                   'ret_desc': 'failed',
                   'dev_addr': '/dev/sdb'}
        context = ''
        image_service = ''
        image_meta = ''

        mock_get_manage_ip.return_value = '127.0.0.1'
        mock_attach.return_value = failure
        self.assertRaises(exception.CinderException,
                          self.driver.copy_volume_to_image,
                          context, self.volume, image_service, image_meta)
        mock_attach.return_value = None
        self.assertRaises(exception.CinderException,
                          self.driver.copy_volume_to_image,
                          context, self.volume, image_service, image_meta)

    @mock.patch.object(dsware.DSWAREDriver, '_get_dsware_manage_ip')
    @mock.patch.object(dsware.DSWAREDriver, '_attach_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_query_volume_attach')
    @mock.patch.object(image_utils, 'upload_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_detach_volume')
    def test_copy_volume_to_image_query_attach_fail(self, mock_detach,
                                                    mock_upload, mock_query,
                                                    mock_attach,
                                                    mock_get_manage_ip):
        already_attached = {'ret_code': '50151401',
                            'ret_desc': 'already_attached',
                            'dev_addr': '/dev/sdb'}
        failure = {'ret_code': '50510011',
                   'ret_desc': 'failed',
                   'dev_addr': '/dev/sdb'}
        context = ''
        image_service = ''
        image_meta = ''

        mock_get_manage_ip.return_value = '127.0.0.1'
        mock_attach.return_value = already_attached
        mock_query.return_value = failure
        self.assertRaises(exception.CinderException,
                          self.driver.copy_volume_to_image,
                          context, self.volume, image_service, image_meta)

        mock_query.return_value = None
        self.assertRaises(exception.CinderException,
                          self.driver.copy_volume_to_image,
                          context, self.volume, image_service, image_meta)

    @mock.patch.object(dsware.DSWAREDriver, '_get_dsware_manage_ip')
    @mock.patch.object(dsware.DSWAREDriver, '_attach_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_query_volume_attach')
    @mock.patch.object(image_utils, 'upload_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_detach_volume')
    def test_copy_volume_to_image_upload_fail(self, mock_detach, mock_upload,
                                              mock_query, mock_attach,
                                              mock_get_manage_ip):
        success = {'ret_code': '0',
                   'ret_desc': 'success',
                   'dev_addr': '/dev/sdb'}
        already_attached = {'ret_code': '50151401',
                            'ret_desc': 'already_attached',
                            'dev_addr': '/dev/sdb'}
        context = ''
        image_service = ''
        image_meta = ''

        mock_get_manage_ip.return_value = '127.0.0.1'
        mock_attach.return_value = already_attached
        mock_query.return_value = success
        mock_upload.side_effect = exception.CinderException(
            'upload_volume error')
        self.assertRaises(exception.CinderException,
                          self.driver.copy_volume_to_image,
                          context, self.volume, image_service, image_meta)

    @mock.patch.object(fspythonapi.FSPythonApi, 'query_volume')
    def test_private_get_volume(self, mock_query):
        result_success = {'result': 0}
        result_not_exist = {'result': "50150005\n"}
        result_exception = {'result': "50510006\n"}

        mock_query.side_effect = [
            result_success, result_not_exist, result_exception]

        retval = self.driver._get_volume(self.volume.name)
        self.assertTrue(retval)

        retval = self.driver._get_volume(self.volume.name)
        self.assertFalse(retval)

        self.assertRaises(exception.CinderException,
                          self.driver._get_volume,
                          self.volume.name)

    @mock.patch.object(fspythonapi.FSPythonApi, 'delete_volume')
    def test_private_delete_volume(self, mock_delete):
        result_success = 0
        result_not_exist = '50150005\n'
        result_being_deleted = '50151002\n'
        result_exception = '51050006\n'

        mock_delete.side_effect = [result_success, result_not_exist,
                                   result_being_deleted, result_exception]

        retval = self.driver._delete_volume(self.volume.name)
        self.assertTrue(retval)

        retval = self.driver._delete_volume(self.volume.name)
        self.assertTrue(retval)

        retval = self.driver._delete_volume(self.volume.name)
        self.assertTrue(retval)

        self.assertRaises(exception.CinderException,
                          self.driver._delete_volume, self.volume.name)

    @mock.patch.object(dsware.DSWAREDriver, '_get_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_delete_volume')
    def test_delete_volume(self, mock_delete, mock_get):
        mock_get.return_value = False
        retval = self.driver.delete_volume(self.volume)
        self.assertTrue(retval)

        mock_get.return_value = True
        mock_delete.return_value = True
        retval = self.driver.delete_volume(self.volume)
        self.assertTrue(retval)

        mock_get.return_value = True
        mock_delete.side_effect = exception.CinderException(
            'delete volume exception')
        self.assertRaises(exception.CinderException,
                          self.driver.delete_volume,
                          self.volume)

        mock_get.side_effect = exception.CinderException(
            'get volume exception')
        self.assertRaises(exception.CinderException,
                          self.driver.delete_volume,
                          self.volume)

    @mock.patch.object(fspythonapi.FSPythonApi, 'query_snap')
    def test_private_get_snapshot(self, mock_query):
        result_success = {'result': 0}
        result_not_found = {'result': "50150006\n"}
        result_exception = {'result': "51050007\n"}
        mock_query.side_effect = [result_success, result_not_found,
                                  result_exception]

        retval = self.driver._get_snapshot(self.snapshot.name)
        self.assertTrue(retval)

        retval = self.driver._get_snapshot(self.snapshot.name)
        self.assertFalse(retval)

        self.assertRaises(exception.CinderException,
                          self.driver._get_snapshot,
                          self.snapshot.name)

    @mock.patch.object(fspythonapi.FSPythonApi, 'create_snapshot')
    def test_private_create_snapshot(self, mock_create):
        mock_create.side_effect = [0, 1]

        self.driver._create_snapshot(self.snapshot.name,
                                     self.volume.name)

        self.assertRaises(exception.CinderException,
                          self.driver._create_snapshot,
                          self.snapshot.name, self.volume.name)

    @mock.patch.object(fspythonapi.FSPythonApi, 'delete_snapshot')
    def test_private_delete_snapshot(self, mock_delete):
        mock_delete.side_effect = [0, 1]

        self.driver._delete_snapshot(self.snapshot.name)

        self.assertRaises(exception.CinderException,
                          self.driver._delete_snapshot, self.snapshot.name)

    @mock.patch.object(dsware.DSWAREDriver, '_get_volume')
    @mock.patch.object(dsware.DSWAREDriver, '_create_snapshot')
    def test_create_snapshot(self, mock_create, mock_get):
        mock_get.return_value = True
        self.driver.create_snapshot(self.snapshot)

        mock_create.side_effect = exception.CinderException(
            'create snapshot failed')
        self.assertRaises(exception.CinderException,
                          self.driver.create_snapshot, self.snapshot)

        mock_get.side_effect = [
            False, exception.CinderException('get volume failed')]
        self.assertRaises(exception.CinderException,
                          self.driver.create_snapshot,
                          self.snapshot)
        self.assertRaises(exception.CinderException,
                          self.driver.create_snapshot,
                          self.snapshot)

    @mock.patch.object(dsware.DSWAREDriver, '_get_snapshot')
    @mock.patch.object(dsware.DSWAREDriver, '_delete_snapshot')
    def test_delete_snapshot(self, mock_delete, mock_get):
        mock_get.side_effect = [True, False, exception.CinderException, True]
        self.driver.delete_snapshot(self.snapshot)
        self.driver.delete_snapshot(self.snapshot)

        self.assertRaises(exception.CinderException,
                          self.driver.delete_snapshot,
                          self.snapshot)
        mock_delete.side_effect = exception.CinderException(
            'delete snapshot exception')
        self.assertRaises(exception.CinderException,
                          self.driver.delete_snapshot,
                          self.snapshot)

    @mock.patch.object(fspythonapi.FSPythonApi, 'query_pool_info')
    def test_private_update_single_pool_info_status(self, mock_query):
        pool_info = {'result': 0,
                     'pool_id': 10,
                     'total_capacity': 10240,
                     'used_capacity': 5120,
                     'alloc_capacity': 7168}
        pool_info_none = {'result': 1}

        mock_query.side_effect = [pool_info, pool_info_none]

        self.driver._update_single_pool_info_status()
        self.assertEqual({'total_capacity_gb': 10.0,
                          'free_capacity_gb': 5.0,
                          'volume_backend_name': None,
                          'vendor_name': 'Open Source',
                          'driver_version': '1.0',
                          'storage_protocol': 'dsware',
                          'reserved_percentage': 0,
                          'QoS_support': False},
                         self.driver._stats)

        self.driver._update_single_pool_info_status()
        self.assertIsNone(self.driver._stats)

    @mock.patch.object(fspythonapi.FSPythonApi, 'query_pool_type')
    def test_private_update_multi_pool_of_same_type_status(self, mock_query):
        query_result = (0, [{'result': 0,
                             'pool_id': '0',
                             'total_capacity': '10240',
                             'used_capacity': '5120',
                             'alloc_capacity': '7168'}])
        query_result_none = (0, [])

        mock_query.side_effect = [query_result, query_result_none]

        self.driver._update_multi_pool_of_same_type_status()
        self.assertEqual({'volume_backend_name': None,
                          'vendor_name': 'Open Source',
                          'driver_version': '1.0',
                          'storage_protocol': 'dsware',
                          'pools': [{'pool_name': '0',
                                     'total_capacity_gb': 10.0,
                                     'allocated_capacity_gb': 5.0,
                                     'free_capacity_gb': 5.0,
                                     'QoS_support': False,
                                     'reserved_percentage': 0}]},
                         self.driver._stats)

        self.driver._update_multi_pool_of_same_type_status()
        self.assertIsNone(self.driver._stats)

    def test_private_calculate_pool_info(self):
        pool_sets = [{'pool_id': 0,
                      'total_capacity': 10240,
                      'used_capacity': 5120,
                      'QoS_support': False,
                      'reserved_percentage': 0}]
        retval = self.driver._calculate_pool_info(pool_sets)
        self.assertEqual([{'pool_name': 0,
                           'total_capacity_gb': 10.0,
                           'allocated_capacity_gb': 5.0,
                           'free_capacity_gb': 5.0,
                           'QoS_support': False,
                           'reserved_percentage': 0}],
                         retval)

    @mock.patch.object(dsware.DSWAREDriver, '_update_single_pool_info_status')
    @mock.patch.object(dsware.DSWAREDriver,
                       '_update_multi_pool_of_same_type_status')
    @mock.patch.object(fspythonapi.FSPythonApi, 'query_dsware_version')
    def test_get_volume_stats(self, mock_query, mock_type, mock_info):
        mock_query.return_value = 1

        self.driver.get_volume_stats(False)
        mock_query.assert_not_called()

        self.driver.get_volume_stats(True)
        mock_query.assert_called_once_with()
